"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/**
 * 低代码引擎的出码模块，负责将编排产出的 Schema 转换成实际可执行的代码。
 * 注意：为了保持 API 的稳定性，这里所有导出的 API 均要显式命名方式导出
 *     （即用 export { xxx } from 'xx' 的方式，不要直接 export * from 'xxx')
 *      而且所有导出的 API 务必在 tests/public 中编写单元测试
 */
const ProjectBuilder_1 = require("../generator/ProjectBuilder");
const ModuleBuilder_1 = require("../generator/ModuleBuilder");
const disk_1 = require("../publisher/disk");
const zip_1 = require("../publisher/zip");
const icejs_1 = tslib_1.__importStar(require("../solutions/icejs"));
const icejs3_1 = tslib_1.__importStar(require("../solutions/icejs3"));
const icejszwh_1 = tslib_1.__importStar(require("../solutions/icejszwh"));
const rax_app_1 = tslib_1.__importStar(require("../solutions/rax-app"));
// 引入说明
const const_1 = require("../plugins/component/react/const");
const generator_1 = require("../const/generator");
// 引入通用插件组
const esmodule_1 = tslib_1.__importDefault(require("../plugins/common/esmodule"));
const requireUtils_1 = tslib_1.__importDefault(require("../plugins/common/requireUtils"));
const styleImport_1 = tslib_1.__importDefault(require("../plugins/common/styleImport"));
const css_1 = tslib_1.__importDefault(require("../plugins/component/style/css"));
const constants_1 = tslib_1.__importDefault(require("../plugins/project/constants"));
const i18n_1 = tslib_1.__importDefault(require("../plugins/project/i18n"));
const utils_1 = tslib_1.__importDefault(require("../plugins/project/utils"));
const prettier_1 = tslib_1.__importDefault(require("../postprocessor/prettier"));
// 引入全局常用工具
const globalUtils = tslib_1.__importStar(require("../utils"));
const CONSTANTS = tslib_1.__importStar(require("../const"));
// 引入内置解决方案模块
const icejs_2 = tslib_1.__importDefault(require("../plugins/project/framework/icejs"));
const icejs3_2 = tslib_1.__importDefault(require("../plugins/project/framework/icejs3"));
const icejszwh_2 = tslib_1.__importDefault(require("../plugins/project/framework/icejszwh"));
const rax_1 = tslib_1.__importDefault(require("../plugins/project/framework/rax"));
exports.default = {
    createProjectBuilder: ProjectBuilder_1.createProjectBuilder,
    createModuleBuilder: ModuleBuilder_1.createModuleBuilder,
    solutions: {
        icejs: icejs_1.default,
        icejs3: icejs3_1.default,
        rax: rax_app_1.default,
        icejszwh: icejszwh_1.default,
    },
    solutionParts: {
        icejs: icejs_2.default,
        icejs3: icejs3_2.default,
        rax: rax_1.default,
        icejszwh: icejszwh_2.default,
    },
    publishers: {
        disk: disk_1.createDiskPublisher,
        zip: zip_1.createZipPublisher,
    },
    plugins: {
        common: {
            /**
             * 处理 ES Module
             * @deprecated please use esModule
             */
            esmodule: esmodule_1.default,
            esModule: esmodule_1.default,
            requireUtils: requireUtils_1.default,
            styleImport: styleImport_1.default,
        },
        style: {
            css: css_1.default,
        },
        project: {
            constants: constants_1.default,
            i18n: i18n_1.default,
            utils: utils_1.default,
        },
        icejs: Object.assign({}, icejs_1.plugins),
        icejs3: Object.assign({}, icejs3_1.plugins),
        rax: Object.assign({}, rax_app_1.plugins),
        icejszwh: Object.assign({}, icejszwh_1.plugins),
        /**
         * @deprecated please use icejs
         */
        react: Object.assign({}, icejs_1.plugins),
    },
    postprocessor: {
        prettier: prettier_1.default,
    },
    utils: globalUtils,
    chunkNames: {
        COMMON_CHUNK_NAME: generator_1.COMMON_CHUNK_NAME,
        CLASS_DEFINE_CHUNK_NAME: generator_1.CLASS_DEFINE_CHUNK_NAME,
        REACT_CHUNK_NAME: const_1.REACT_CHUNK_NAME,
        FC_DEFINE_CHUNK_NAME: generator_1.FUN_DEFINE_CHUNK_NAME,
    },
    defaultLinkAfter: {
        COMMON_DEFAULT_LINK_AFTER: generator_1.DEFAULT_LINK_AFTER,
    },
    constants: CONSTANTS,
};
// 一些类型定义
tslib_1.__exportStar(require("../types"), exports);
// 一些常量定义
tslib_1.__exportStar(require("../const"), exports);
// 一些工具函数
tslib_1.__exportStar(require("../analyzer/componentAnalyzer"), exports);
tslib_1.__exportStar(require("../parser/SchemaParser"), exports);
tslib_1.__exportStar(require("../generator/ChunkBuilder"), exports);
tslib_1.__exportStar(require("../generator/CodeBuilder"), exports);
tslib_1.__exportStar(require("../generator/ModuleBuilder"), exports);
tslib_1.__exportStar(require("../generator/ProjectBuilder"), exports);
//# sourceMappingURL=index.js.map