"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createModuleBuilder = void 0;
const tslib_1 = require("tslib");
const types_1 = require("../types");
const generator_1 = require("../const/generator");
const SchemaParser_1 = require("../parser/SchemaParser");
const ChunkBuilder_1 = require("./ChunkBuilder");
const CodeBuilder_1 = require("./CodeBuilder");
const resultHelper_1 = require("../utils/resultHelper");
function createModuleBuilder(options = {
    plugins: [],
    postProcessors: [],
}) {
    const chunkGenerator = new ChunkBuilder_1.ChunkBuilder(options.plugins);
    const linker = new CodeBuilder_1.CodeBuilder();
    const generateModule = (input) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        const moduleMainName = options.mainFileName || generator_1.COMMON_SUB_MODULE_NAME;
        if (chunkGenerator.getPlugins().length <= 0) {
            throw new types_1.CodeGeneratorError('No plugins found. Component generation cannot work without any plugins!');
        }
        let files = [];
        const { chunks } = yield chunkGenerator.run(input, {
            ir: input,
            chunks: [],
            depNames: [],
            contextData: options.contextData || {},
        });
        chunks.forEach((fileChunkList) => {
            const content = linker.link(fileChunkList);
            const file = (0, resultHelper_1.createResultFile)(fileChunkList[0].subModule || moduleMainName, fileChunkList[0].fileType, content);
            files.push(file);
        });
        if (options.postProcessors.length > 0) {
            files = files.map((file) => {
                let { content, ext: type, name } = file;
                options.postProcessors.forEach((processer) => {
                    content = processer(content, type, name);
                });
                return (0, resultHelper_1.createResultFile)(file.name, type, content);
            });
        }
        return {
            files,
        };
    });
    const generateModuleCode = (schema) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        // Init
        const schemaParser = new SchemaParser_1.SchemaParser();
        const parseResult = schemaParser.parse(schema);
        const containerInfo = parseResult.containers[0];
        const { files } = yield generateModule(containerInfo);
        const dir = (0, resultHelper_1.createResultDir)(containerInfo.moduleName);
        files.forEach((file) => (0, resultHelper_1.addFile)(dir, file));
        return dir;
    });
    const linkCodeChunks = (chunks, fileName) => {
        const files = [];
        Object.keys(chunks).forEach((fileKey) => {
            const fileChunkList = chunks[fileKey];
            const content = linker.link(fileChunkList);
            const file = (0, resultHelper_1.createResultFile)(fileChunkList[0].subModule || fileName, fileChunkList[0].fileType, content);
            files.push(file);
        });
        return files;
    };
    return {
        chunkGenerator,
        generateModule,
        generateModuleCode,
        linkCodeChunks,
        addPlugin: chunkGenerator.addPlugin.bind(chunkGenerator),
    };
}
exports.createModuleBuilder = createModuleBuilder;
//# sourceMappingURL=ModuleBuilder.js.map