"use strict";
/* eslint-disable @typescript-eslint/indent */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const lowcode_types_1 = require("@alilc/lowcode-types");
const change_case_1 = tslib_1.__importDefault(require("change-case"));
const generator_1 = require("../../../const/generator");
const Scope_1 = require("../../../utils/Scope");
const types_1 = require("../../../types");
const compositeType_1 = require("../../../utils/compositeType");
const expressionParser_1 = require("../../../utils/expressionParser");
const schema_1 = require("../../../utils/schema");
const common_1 = require("../../../utils/common");
const pluginFactory = (config) => {
    const cfg = Object.assign(Object.assign({}, config), { fileType: (config === null || config === void 0 ? void 0 : config.fileType) || types_1.FileType.TSX });
    const plugin = (pre) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const next = Object.assign({}, pre);
        const scope = Scope_1.Scope.createRootScope();
        const dataSourceConfig = (0, schema_1.isValidContainerType)(pre.ir)
            ? pre.ir.dataSource
            : null;
        const dataSourceItems = (dataSourceConfig && dataSourceConfig.list) || [];
        const dataSourceEngineOptions = { runtimeConfig: true };
        if (dataSourceItems.length > 0) {
            const requestHandlersMap = {};
            dataSourceItems.forEach((ds) => {
                var _a, _b;
                const dsType = ds.type || 'fetch';
                if (!(dsType in requestHandlersMap) && dsType !== 'custom') {
                    const handlerFactoryName = `__$$create${change_case_1.default.pascal(dsType)}RequestHandler`;
                    requestHandlersMap[dsType] = {
                        type: 'JSExpression',
                        value: handlerFactoryName +
                            (dsType === 'urlParams' ? '(window.location.search)' : '()'),
                    };
                    const handlerFactoryExportName = `create${change_case_1.default.pascal(dsType)}Handler`;
                    const handlerPkgName = ((_b = (_a = cfg.datasourceConfig) === null || _a === void 0 ? void 0 : _a.handlersPackages) === null || _b === void 0 ? void 0 : _b[dsType]) ||
                        `@alilc/lowcode-datasource-${change_case_1.default.kebab(dsType)}-handler`;
                    next.chunks.push({
                        type: types_1.ChunkType.STRING,
                        fileType: cfg.fileType,
                        name: generator_1.COMMON_CHUNK_NAME.ExternalDepsImport,
                        content: `
              import { ${handlerFactoryExportName} as ${handlerFactoryName} } from '${handlerPkgName}';
            `,
                        linkAfter: [],
                    });
                }
            });
            Object.assign(dataSourceEngineOptions, { requestHandlersMap });
            next.chunks.push({
                type: types_1.ChunkType.STRING,
                fileType: cfg.fileType,
                name: generator_1.COMMON_CHUNK_NAME.ExternalDepsImport,
                content: `
          import { create as __$$createDataSourceEngine } from '@alilc/lowcode-datasource-engine/runtime';
        `,
                linkAfter: [],
            });
            next.chunks.push({
                type: types_1.ChunkType.STRING,
                fileType: cfg.fileType,
                name: generator_1.FC_DEFINE_CHUNK_NAME.InsVar,
                content: `
          _dataSourceConfig = this._defineDataSourceConfig();
          _dataSourceEngine = __$$createDataSourceEngine(
            this._dataSourceConfig,
            this,
            ${(0, compositeType_1.generateCompositeType)(dataSourceEngineOptions, scope)}
          );

          get dataSourceMap() {
            return this._dataSourceEngine.dataSourceMap || {};
          }

          reloadDataSource = async () => {
            await this._dataSourceEngine.reloadDataSource();
          }

          `,
                linkAfter: [...generator_1.DEFAULT_LINK_AFTER[generator_1.FC_DEFINE_CHUNK_NAME.InsVar]],
            });
            next.chunks.unshift({
                type: types_1.ChunkType.STRING,
                fileType: cfg.fileType,
                name: generator_1.FUN_DEFINE_CHUNK_NAME.EffectHookContent,
                content: `
          this._dataSourceEngine.reloadDataSource();
        `,
                linkAfter: [
                    ...generator_1.DEFAULT_LINK_AFTER[generator_1.FUN_DEFINE_CHUNK_NAME.EffectHookContent],
                ],
            });
            next.chunks.push({
                type: types_1.ChunkType.STRING,
                fileType: cfg.fileType,
                name: generator_1.FC_DEFINE_CHUNK_NAME.InsMethod,
                content: `
  _defineDataSourceConfig() {
    const _this = this;
    return (${(0, compositeType_1.generateCompositeType)(Object.assign(Object.assign({}, dataSourceConfig), { list: [
                        ...dataSourceItems.map((item) => (Object.assign(Object.assign({}, item), { isInit: wrapAsFunction(item.isInit, scope), options: wrapAsFunction(item.options, scope) }))),
                    ] }), scope, {
                    handlers: {
                        function: (jsFunc) => (0, expressionParser_1.parseExpressionConvertThis2Context)(jsFunc.value, '_this'),
                        expression: (jsExpr) => (0, expressionParser_1.parseExpressionConvertThis2Context)(jsExpr.value, '_this'),
                    },
                })});
  }
        `,
                linkAfter: [...generator_1.DEFAULT_LINK_AFTER[generator_1.CLASS_DEFINE_CHUNK_NAME.InsMethod]],
            });
        }
        return next;
    });
    return plugin;
};
exports.default = pluginFactory;
function wrapAsFunction(value, scope) {
    if ((0, lowcode_types_1.isJSExpression)(value) || (0, lowcode_types_1.isJSFunction)(value) || (0, common_1.isJSExpressionFn)(value)) {
        return {
            type: 'JSExpression',
            value: `function(){ return ((${value.value}))}.bind(this)`,
        };
    }
    return {
        type: 'JSExpression',
        value: `function(){return((${(0, compositeType_1.generateCompositeType)(value, scope)}))}.bind(this)`,
    };
}
//# sourceMappingURL=containerInjectDataSourceEngine.js.map