"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/* eslint-disable @typescript-eslint/indent */
const generator_1 = require("../../../const/generator");
const types_1 = require("../../../types");
const const_1 = require("./const");
const const_2 = require("../../../const");
const pluginFactory = (config) => {
    const cfg = Object.assign({ fileType: types_1.FileType.JSX }, config);
    const plugin = (pre) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        var _a;
        const next = Object.assign({}, pre);
        const ir = next.ir;
        const useRef = !!((_a = ir.analyzeResult) === null || _a === void 0 ? void 0 : _a.isUsingRef);
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: cfg.fileType,
            name: generator_1.COMMON_CHUNK_NAME.InternalDepsImport,
            content: "import __$$constants from '../../constants';",
            linkAfter: [generator_1.COMMON_CHUNK_NAME.ExternalDepsImport],
        });
        // TODO: i18n 是可选的，如果没有 i18n 这个文件怎么办？该怎么判断？
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: cfg.fileType,
            name: generator_1.COMMON_CHUNK_NAME.InternalDepsImport,
            content: "import * as __$$i18n from '../../i18n';",
            linkAfter: [generator_1.COMMON_CHUNK_NAME.ExternalDepsImport],
        });
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: cfg.fileType,
            name: generator_1.CLASS_DEFINE_CHUNK_NAME.ConstructorContent,
            content: `
        __$$i18n._inject2(this);
      `,
            linkAfter: [...const_2.DEFAULT_LINK_AFTER[generator_1.CLASS_DEFINE_CHUNK_NAME.ConstructorContent]],
        });
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: cfg.fileType,
            name: generator_1.CLASS_DEFINE_CHUNK_NAME.InsVar,
            content: `
        _context = this._createContext();
      `,
            linkAfter: [generator_1.CLASS_DEFINE_CHUNK_NAME.Start],
        });
        // TODO: 按照目前的实现方案，代码的插拔能力太弱了，需要有一些变化。
        //   Step 1: 增加前置的分析器
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: cfg.fileType,
            name: generator_1.CLASS_DEFINE_CHUNK_NAME.InsPrivateMethod,
            content: `
        _createContext() {
          const self = this;
          const context = {
            get state() {
              return self.state;
            },
            setState(newState, callback) {
              self.setState(newState, callback);
            },
            get dataSourceMap() {
              return self._dataSourceEngine.dataSourceMap || {};
            },
            async reloadDataSource() {
              await self._dataSourceEngine.reloadDataSource();
            },
            get utils() {
              return self._utils;
            },
            get page() {
              return context;
            },
            get component() {
              return context;
            },
            get props() {
              return self.props;
            },
            get constants() {
              return __$$constants;
            },
            i18n: __$$i18n.i18n,
            i18nFormat: __$$i18n.i18nFormat,
            getLocale: __$$i18n.getLocale,
            setLocale(locale) {
              __$$i18n.setLocale(locale);
              self.forceUpdate();
            },${useRef
                ? `
                  $(refName) {
                    return self._refsManager.get(refName);
                  },
                  $$(refName) {
                    return self._refsManager.getAll(refName);
                  },
                  get _refsManager() {
                    if (!self._refsManager) {
                      self._refsManager = new RefsManager();
                    }
                    return self._refsManager;
                  },
                `
                : ''}
            ...this._methods,
          };

          return context;
        }
      `,
            linkAfter: [const_1.RAX_CHUNK_NAME.ClassRenderEnd],
        });
        return next;
    });
    return plugin;
};
exports.default = pluginFactory;
//# sourceMappingURL=containerInjectContext.js.map