"use strict";
/* eslint-disable @typescript-eslint/indent */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const lowcode_types_1 = require("@alilc/lowcode-types");
const change_case_1 = tslib_1.__importDefault(require("change-case"));
const generator_1 = require("../../../const/generator");
const Scope_1 = require("../../../utils/Scope");
const types_1 = require("../../../types");
const compositeType_1 = require("../../../utils/compositeType");
const expressionParser_1 = require("../../../utils/expressionParser");
const schema_1 = require("../../../utils/schema");
const const_1 = require("./const");
const pluginFactory = (config) => {
    var _a;
    const cfg = Object.assign(Object.assign({ fileType: types_1.FileType.JSX }, config), { dataSourceHandlersPackageMap: (config === null || config === void 0 ? void 0 : config.dataSourceHandlersPackageMap) || ((_a = config === null || config === void 0 ? void 0 : config.datasourceConfig) === null || _a === void 0 ? void 0 : _a.handlersPackages) });
    const plugin = (pre) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        var _b;
        const next = Object.assign({}, pre);
        const scope = Scope_1.Scope.createRootScope();
        const dataSourceConfig = (0, schema_1.isContainerSchema)(pre.ir) ? pre.ir.dataSource : null;
        const dataSourceItems = (dataSourceConfig && dataSourceConfig.list) || [];
        const dataSourceEngineOptions = { runtimeConfig: true };
        if (dataSourceItems.length > 0) {
            const requestHandlersMap = {};
            dataSourceItems.forEach((ds) => {
                var _a;
                const dsType = ds.type || 'fetch';
                if (!(dsType in requestHandlersMap) && dsType !== 'custom') {
                    const handlerFactoryName = `__$$create${change_case_1.default.pascal(dsType)}RequestHandler`;
                    requestHandlersMap[dsType] = {
                        type: 'JSExpression',
                        value: `${handlerFactoryName}(${dsType === 'urlParams' ? '__$$getSearchParams()' : ''})`,
                    };
                    const handlerFactoryExportName = `create${change_case_1.default.pascal(dsType)}Handler`;
                    const handlerPkgName = ((_a = cfg.dataSourceHandlersPackageMap) === null || _a === void 0 ? void 0 : _a[dsType]) ||
                        `@alilc/lowcode-datasource-${change_case_1.default.kebab(dsType)}-handler`;
                    next.chunks.push({
                        type: types_1.ChunkType.STRING,
                        fileType: types_1.FileType.JSX,
                        name: generator_1.COMMON_CHUNK_NAME.ExternalDepsImport,
                        content: `
              import { ${handlerFactoryExportName} as ${handlerFactoryName} } from '${handlerPkgName}';
            `,
                        linkAfter: [],
                    });
                }
            });
            Object.assign(dataSourceEngineOptions, { requestHandlersMap });
        }
        const datasourceEnginePackageName = ((_b = cfg.datasourceConfig) === null || _b === void 0 ? void 0 : _b.enginePackage) || '@alilc/lowcode-datasource-engine';
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: types_1.FileType.JSX,
            name: generator_1.COMMON_CHUNK_NAME.ExternalDepsImport,
            content: `
        import { create as __$$createDataSourceEngine } from '${datasourceEnginePackageName}/runtime';
      `,
            linkAfter: [],
        });
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: cfg.fileType,
            name: generator_1.CLASS_DEFINE_CHUNK_NAME.InsVar,
            content: `
        _dataSourceConfig = this._defineDataSourceConfig();
        _dataSourceEngine = __$$createDataSourceEngine(
          this._dataSourceConfig,
          this._context,
          ${(0, compositeType_1.generateCompositeType)(dataSourceEngineOptions, scope)}
        );`,
            linkAfter: [generator_1.CLASS_DEFINE_CHUNK_NAME.Start],
        });
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: cfg.fileType,
            name: const_1.RAX_CHUNK_NAME.ClassDidMountContent,
            content: `
        this._dataSourceEngine.reloadDataSource();
      `,
            linkAfter: [const_1.RAX_CHUNK_NAME.ClassDidMountBegin],
        });
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: cfg.fileType,
            name: generator_1.CLASS_DEFINE_CHUNK_NAME.InsPrivateMethod,
            content: `
_defineDataSourceConfig() {
  const __$$context = this._context;
  return (${(0, compositeType_1.generateCompositeType)(Object.assign(Object.assign({}, dataSourceConfig), { list: [
                    ...dataSourceItems.map((item) => {
                        var _a;
                        return (Object.assign(Object.assign({ 
                            // 数据源引擎默认的 errorHandler 是空的，而且并不会触发组件重新渲染……
                            // 这会导致页面状态不能正常展示，故这里处理下:
                            errorHandler: {
                                type: 'JSFunction',
                                value: `function (err){
              setTimeout(() => {
                this.setState({ __refresh: Date.now() + Math.random() });
              }, 0);
              throw err;
            }`,
                            } }, item), { isInit: typeof item.isInit === 'boolean' || typeof item.isInit === 'undefined'
                                ? (_a = item.isInit) !== null && _a !== void 0 ? _a : true
                                : wrapAsFunction(item.isInit, scope), options: wrapAsFunction(item.options, scope) }));
                    }),
                ] }), scope, {
                handlers: {
                    function: (jsFunc) => (0, expressionParser_1.parseExpressionConvertThis2Context)(jsFunc.value, '__$$context'),
                    expression: (jsExpr) => (0, expressionParser_1.parseExpressionConvertThis2Context)(jsExpr.value, '__$$context'),
                },
            })});
}
      `,
            linkAfter: [const_1.RAX_CHUNK_NAME.ClassRenderEnd],
        });
        return next;
    });
    return plugin;
};
exports.default = pluginFactory;
function wrapAsFunction(value, scope) {
    if ((0, lowcode_types_1.isJSExpression)(value) || (0, lowcode_types_1.isJSFunction)(value)) {
        return {
            type: 'JSExpression',
            value: `function(){ return ((${value.value}))}`,
        };
    }
    return {
        type: 'JSExpression',
        value: `function(){return((${(0, compositeType_1.generateCompositeType)(value, scope)}))}`,
    };
}
//# sourceMappingURL=containerInjectDataSourceEngine.js.map