"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const lowcode_types_1 = require("@alilc/lowcode-types");
const lodash_1 = tslib_1.__importDefault(require("lodash"));
const change_case_1 = tslib_1.__importDefault(require("change-case"));
const types_1 = require("../../../types");
const const_1 = require("./const");
const generator_1 = require("../../../const/generator");
const jsExpression_1 = require("../../../utils/jsExpression");
const nodeToJSX_1 = require("../../../utils/nodeToJSX");
const compositeType_1 = require("../../../utils/compositeType");
const Scope_1 = require("../../../utils/Scope");
const expressionParser_1 = require("../../../utils/expressionParser");
const transformThis2Context_1 = require("../../../core/jsx/handlers/transformThis2Context");
const transformJsExpression_1 = require("../../../core/jsx/handlers/transformJsExpression");
// TODO: componentName 若并非大写字符打头，甚至并非是一个有效的 JS 标识符怎么办？？
// FIXME: 我想了下，这块应该放到解析阶段就去做掉，对所有 componentName 做 identifier validate，然后对不合法的做统一替换。
const pluginFactory = (config) => {
    const cfg = Object.assign({ fileType: types_1.FileType.JSX }, config);
    const plugin = (pre) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const next = Object.assign({}, pre);
        const ir = next.ir;
        const rootScope = Scope_1.Scope.createRootScope();
        const { tolerateEvalErrors = true, evalErrorsHandler = '' } = next.contextData;
        // Rax 构建到小程序的时候，不能给组件起起别名，得直接引用，故这里将所有的别名替换掉
        // 先收集下所有的 alias 的映射
        const componentsNameAliasMap = new Map();
        next.chunks.forEach((chunk) => {
            if (isImportAliasDefineChunk(chunk)) {
                componentsNameAliasMap.set(chunk.ext.aliasName, chunk.ext.originalName);
            }
        });
        // 注意：这里其实隐含了一个假设：schema 中的 componentName 应该是一个有效的 JS 标识符，而且是大写字母打头的
        // FIXME: 为了快速修复临时加的逻辑，需要用 pre-process 的方式替代处理。
        const mapComponentNameToAliasOrKeepIt = (componentName) => componentsNameAliasMap.get(componentName) || componentName;
        // 然后过滤掉所有的别名 chunks
        next.chunks = next.chunks.filter((chunk) => !isImportAliasDefineChunk(chunk));
        // 如果直接按目前的 React 的方式之间出码 JSX 的话，会有 3 个问题：
        // 1. 小程序出码的时候，循环变量没法拿到
        // 2. 小程序出码的时候，很容易出现 Uncaught TypeError: Cannot read property 'avatar' of undefined 这样的异常(如下图的 50 行) -- 因为若直接出码，Rax 构建到小程序的时候会立即计算所有在视图中用到的变量
        // 3. 通过 this.xxx 能拿到的东西太多了，而且自定义的 methods 可能会无意间破坏 Rax 框架或小程序框架在页面 this 上的东东
        const customHandlers = {
            expression(input, scope) {
                return (0, transformJsExpression_1.transformJsExpr)((0, jsExpression_1.generateExpression)(input, scope), scope, {
                    dontWrapEval: !tolerateEvalErrors,
                });
            },
            function(input, scope) {
                return (0, transformThis2Context_1.transformThis2Context)(input.value || 'null', scope);
            },
        };
        // 创建代码生成器
        const commonNodeGenerator = (0, nodeToJSX_1.createNodeGenerator)({
            handlers: customHandlers,
            tagMapping: mapComponentNameToAliasOrKeepIt,
            nodePlugins: [nodeToJSX_1.generateReactExprInJS, nodeToJSX_1.generateConditionReactCtrl, generateRaxLoopCtrl],
            attrPlugins: [generateNodeAttrForRax.bind({ cfg })],
        });
        // 生成 JSX 代码
        const jsxContent = commonNodeGenerator(ir, rootScope);
        if (!cfg.ignoreMiniApp) {
            next.chunks.push({
                type: types_1.ChunkType.STRING,
                fileType: cfg.fileType,
                name: generator_1.COMMON_CHUNK_NAME.ExternalDepsImport,
                content: "import { isMiniApp as __$$isMiniApp } from 'universal-env';",
                linkAfter: [],
            });
        }
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: cfg.fileType,
            name: const_1.RAX_CHUNK_NAME.ClassRenderPre,
            // TODO: setState, dataSourceMap, reloadDataSource, utils, i18n, i18nFormat, getLocale, setLocale 这些在 Rax 的编译模式下不能在视图中直接访问，需要转化成 this.xxx
            content: `
        const __$$context = this._context;
        const { state, setState, dataSourceMap, reloadDataSource, utils, constants, i18n, i18nFormat, getLocale, setLocale } = __$$context;
      `,
            linkAfter: [const_1.RAX_CHUNK_NAME.ClassRenderBegin],
        });
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: cfg.fileType,
            name: const_1.RAX_CHUNK_NAME.ClassRenderJSX,
            content: `return ${jsxContent};`,
            linkAfter: [const_1.RAX_CHUNK_NAME.ClassRenderBegin, const_1.RAX_CHUNK_NAME.ClassRenderPre],
        });
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: cfg.fileType,
            name: generator_1.COMMON_CHUNK_NAME.CustomContent,
            content: [
                tolerateEvalErrors &&
                    `
        function __$$eval(expr) {
          try {
            return expr();
          } catch (error) {
            ${evalErrorsHandler}
          }
        }

        function __$$evalArray(expr) {
          const res = __$$eval(expr);
          return Array.isArray(res) ? res : [];
        }
        `,
                `
        function __$$createChildContext(oldContext, ext) {
          return Object.assign({}, oldContext, ext);
        }
      `,
            ]
                .filter(Boolean)
                .join('\n'),
            linkAfter: [generator_1.COMMON_CHUNK_NAME.FileExport],
        });
        return next;
        function generateRaxLoopCtrl(nodeItem, scope, config, next) {
            var _a, _b;
            if (nodeItem.loop) {
                const loopItemName = ((_a = nodeItem.loopArgs) === null || _a === void 0 ? void 0 : _a[0]) || 'item';
                const loopIndexName = ((_b = nodeItem.loopArgs) === null || _b === void 0 ? void 0 : _b[1]) || 'index';
                const subScope = scope.createSubScope([loopItemName, loopIndexName]);
                const pieces = next ? next(nodeItem, subScope, config) : [];
                const loopDataExpr = tolerateEvalErrors
                    ? `__$$evalArray(() => (${(0, transformThis2Context_1.transformThis2Context)((0, compositeType_1.generateCompositeType)(nodeItem.loop, scope, { handlers: config === null || config === void 0 ? void 0 : config.handlers }), scope)}))`
                    : `(${(0, transformThis2Context_1.transformThis2Context)((0, compositeType_1.generateCompositeType)(nodeItem.loop, scope, { handlers: config === null || config === void 0 ? void 0 : config.handlers }), scope)})`;
                pieces.unshift({
                    value: `${loopDataExpr}.map((${loopItemName}, ${loopIndexName}) => ((__$$context) => (`,
                    type: types_1.PIECE_TYPE.BEFORE,
                });
                pieces.push({
                    value: `))(__$$createChildContext(__$$context, { ${loopItemName}, ${loopIndexName} })))`,
                    type: types_1.PIECE_TYPE.AFTER,
                });
                return pieces;
            }
            return next ? next(nodeItem, scope, config) : [];
        }
    });
    return plugin;
};
exports.default = pluginFactory;
function isImportAliasDefineChunk(chunk) {
    var _a;
    return (chunk.name === generator_1.COMMON_CHUNK_NAME.ImportAliasDefine &&
        !!chunk.ext &&
        typeof chunk.ext.aliasName === 'string' &&
        typeof chunk.ext.originalName === 'string' &&
        !!((_a = chunk.ext.dependency) === null || _a === void 0 ? void 0 : _a.componentName));
}
function generateNodeAttrForRax(attrData, scope, config, next) {
    if (!this.cfg.ignoreMiniApp && /^on/.test(attrData.attrName)) {
        // else: onXxx 的都是事件处理函数需要特殊处理下
        return generateEventHandlerAttrForRax(attrData.attrName, attrData.attrValue, scope, config);
    }
    if (attrData.attrName === 'ref') {
        return [
            {
                name: attrData.attrName,
                value: `__$$context._refsManager.linkRef('${attrData.attrValue}')`,
                type: types_1.PIECE_TYPE.ATTR,
            },
        ];
    }
    return next ? next(attrData, scope, config) : [];
}
function generateEventHandlerAttrForRax(attrName, attrValue, scope, config) {
    var _a;
    // -- 事件处理函数中 JSExpression 转成 JSFunction 来处理，避免当 JSExpression 处理的时候多包一层 eval 而导致 Rax 转码成小程序的时候出问题
    const valueExpr = (0, compositeType_1.generateCompositeType)((0, lowcode_types_1.isJSExpression)(attrValue) ? { type: 'JSFunction', value: attrValue.value } : attrValue, scope, {
        handlers: config === null || config === void 0 ? void 0 : config.handlers,
    });
    // 查询当前作用域下的变量
    const currentScopeVariables = ((_a = scope.bindings) === null || _a === void 0 ? void 0 : _a.getAllBindings()) || [];
    if (currentScopeVariables.length <= 0) {
        return [
            {
                type: types_1.PIECE_TYPE.ATTR,
                name: attrName,
                value: valueExpr,
            },
        ];
    }
    // 提取出所有的未定义的全局变量
    const undeclaredVariablesInValueExpr = (0, expressionParser_1.parseExpressionGetGlobalVariables)(valueExpr);
    const referencedLocalVariables = lodash_1.default.intersection(undeclaredVariablesInValueExpr, currentScopeVariables);
    if (referencedLocalVariables.length <= 0) {
        return [
            {
                type: types_1.PIECE_TYPE.ATTR,
                name: attrName,
                value: valueExpr,
            },
        ];
    }
    const wrappedAttrValueExpr = [
        '(...__$$args) => {',
        '  if (__$$isMiniApp) {',
        '    const __$$event = __$$args[0];',
        ...referencedLocalVariables.map((localVar) => `const ${localVar} = __$$event.target.dataset.${localVar};`),
        `    return (${valueExpr}).apply(this, __$$args);`,
        '  } else {',
        `    return (${valueExpr}).apply(this, __$$args);`,
        '  }',
        '}',
    ].join('\n');
    return [
        ...referencedLocalVariables.map((localVar) => ({
            type: types_1.PIECE_TYPE.ATTR,
            name: `data-${change_case_1.default.snake(localVar)}`,
            value: localVar,
        })),
        {
            type: types_1.PIECE_TYPE.ATTR,
            name: attrName,
            value: wrappedAttrValueExpr,
        },
    ];
}
//# sourceMappingURL=jsx.js.map