"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const types_1 = require("../../../../../types");
const generator_1 = require("../../../../../const/generator");
const format_1 = require("../../../../../utils/format");
const theme_1 = require("../../../../../utils/theme");
function getContent(cfg, routesContent) {
    return `
import { join } from 'path';
import { defineConfig } from '@ice/app';
import _ from 'lodash';
import fusion from '@ice/plugin-fusion';
import locales from '@ice/plugin-moment-locales';
import type { Plugin } from '@ice/app/esm/types';

interface PluginOptions {
  id: string;
}

const plugin: Plugin<PluginOptions> = (options) => ({
  // name 可选，插件名称
  name: 'plugin-name',
  // setup 必选，用于定制工程构建配置
  setup: ({ onGetConfig, modifyUserConfig }) => {
    modifyUserConfig('codeSplitting', 'page');

    onGetConfig((config) => {
      config.entry = {
        web: join(process.cwd(), '.ice/entry.client.tsx'),
      };

      config.cssFilename = '[name].css';

      config.configureWebpack = config.configureWebpack || [];
      config.configureWebpack?.push((webpackConfig) => {
        if (webpackConfig.output) {
          webpackConfig.output.filename = '[name].js';
          webpackConfig.output.chunkFilename = '[name].js';
        }
        return webpackConfig;
      });

      config.swcOptions = _.merge(config.swcOptions, {
        compilationConfig: {
          jsc: {
            transform: {
              react: {
                runtime: 'classic',
              },
            },
          },
        }
      });

      // 解决 webpack publicPath 问题
      config.transforms = config.transforms || [];
      config.transforms.push((source: string, id: string) => {
        if (id.includes('.ice/entry.client.tsx')) {
          let code = \`
          if (!__webpack_public_path__?.startsWith('http') && document.currentScript) {
            // @ts-ignore
            __webpack_public_path__ = document.currentScript.src.replace(/^(.*\\\\/)[^/]+$/, '$1');
            window.__ICE_ASSETS_MANIFEST__ = window.__ICE_ASSETS_MANIFEST__ || {};
            window.__ICE_ASSETS_MANIFEST__.publicPath = __webpack_public_path__;
          }
          \`;
          code += source;
          return { code };
        }
      });
    });
  },
});

// The project config, see https://v3.ice.work/docs/guide/basic/config
const minify = process.env.NODE_ENV === 'production' ? 'swc' : false;
export default defineConfig(() => ({
  ssr: false,
  ssg: false,
  minify,
  ${routesContent}
  externals: {
    react: 'React',
    'react-dom': 'ReactDOM',
    'react-dom/client': 'ReactDOM',
    '@alifd/next': 'Next',
    lodash: 'var window._',
    '@alilc/lowcode-engine': 'var window.AliLowCodeEngine',
  },
  plugins: [
    fusion(${(cfg === null || cfg === void 0 ? void 0 : cfg.themePackage) ? `{
      importStyle: 'sass',
      themePackage: '${(0, theme_1.getThemeInfo)(cfg.themePackage).name}',
    }` : `{
      importStyle: 'sass',
    }`}),
    locales(),
    plugin(),
  ]
}));
  `;
}
function getRoutesContent(navData, needShell = true) {
    const routes = [
        'routes: {',
        '  defineRoutes: route => {',
    ];
    function _getRoutes(nav, _routes = []) {
        const { slug, children } = nav;
        if (children && children.length > 0) {
            children.forEach((_nav) => _getRoutes(_nav, _routes));
        }
        else if (slug) {
            _routes.push(`route('/${slug}', '${slug}/index.jsx');`);
        }
    }
    if (needShell) {
        routes.push("    route('/', 'layout.jsx', () => {");
    }
    navData === null || navData === void 0 ? void 0 : navData.forEach((nav) => {
        _getRoutes(nav, routes);
    });
    if (needShell) {
        routes.push('    });');
    }
    routes.push('  }'); // end of defineRoutes
    routes.push('  },'); // end of routes
    return routes.join('\n');
}
const pluginFactory = (cfg) => {
    const plugin = (pre) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const next = Object.assign({}, pre);
        const { navConfig } = next.contextData;
        const routesContent = (navConfig === null || navConfig === void 0 ? void 0 : navConfig.data) ? getRoutesContent(navConfig.data, true) : '';
        next.chunks.push({
            type: types_1.ChunkType.STRING,
            fileType: types_1.FileType.MTS,
            name: generator_1.COMMON_CHUNK_NAME.FileMainContent,
            content: (0, format_1.format)(getContent(cfg, routesContent)),
            linkAfter: [],
        });
        return next;
    });
    return plugin;
};
exports.default = pluginFactory;
//# sourceMappingURL=buildConfig.js.map