"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const generator_1 = require("../../../../../const/generator");
const types_1 = require("../../../../../types");
const schema_1 = require("../../../../../utils/schema");
const errors_1 = require("../../../../../utils/errors");
const version_1 = require("../../../../../utils/version");
const dataSource_1 = require("../../../../../utils/dataSource");
const pluginFactory = (cfg) => {
    const plugin = (pre) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const next = Object.assign({}, pre);
        const ir = next.ir;
        const npmDeps = getNpmDependencies(ir);
        const packageJson = {
            name: (cfg === null || cfg === void 0 ? void 0 : cfg.packageName) || 'rax-demo-app',
            version: (cfg === null || cfg === void 0 ? void 0 : cfg.packageVersion) || '1.0.0',
            scripts: {
                start: 'rax-app start',
                build: 'rax-app build',
                eslint: 'eslint --ext .js,.jsx ./',
                stylelint: 'stylelint "**/*.{css,scss,less}"',
                prettier: 'prettier **/* --write',
                lint: 'npm run eslint && npm run stylelint',
            },
            dependencies: Object.assign(Object.assign(Object.assign({}, (0, dataSource_1.buildDataSourceDependencies)(ir, cfg === null || cfg === void 0 ? void 0 : cfg.datasourceConfig)), { 
                // 环境判断
                'universal-env': '^3.2.0', 
                // 国际化相关依赖:
                'intl-messageformat': '^9.3.6', 
                // 基础库
                rax: '^1.1.0', 'rax-document': '^0.1.6' }), npmDeps.reduce((acc, npm) => (Object.assign(Object.assign({}, acc), { [npm.package]: npm.version || '*' })), {})),
            devDependencies: {
                '@iceworks/spec': '^1.0.0',
                'rax-app': '^3.0.0',
                eslint: '^6.8.0',
                prettier: '^2.1.2',
                stylelint: '^13.7.2',
            },
        };
        next.chunks.push({
            type: types_1.ChunkType.JSON,
            fileType: types_1.FileType.JSON,
            name: generator_1.COMMON_CHUNK_NAME.FileMainContent,
            content: packageJson,
            linkAfter: [],
        });
        return next;
    });
    return plugin;
};
exports.default = pluginFactory;
function getNpmDependencies(project) {
    const npmDeps = [];
    const npmNameToPkgMap = new Map();
    const allDeps = project.packages;
    allDeps.forEach((dep) => {
        if (!(0, schema_1.isNpmInfo)(dep)) {
            return;
        }
        const existing = npmNameToPkgMap.get(dep.package);
        if (!existing) {
            npmNameToPkgMap.set(dep.package, dep);
            npmDeps.push(dep);
            return;
        }
        if (existing.version !== dep.version) {
            try {
                npmNameToPkgMap.set(dep.package, Object.assign(Object.assign({}, existing), { version: (0, version_1.calcCompatibleVersion)(existing.version, dep.version) }));
            }
            catch (e) {
                throw new Error(`Cannot find compatible version for ${dep.package}. Detail: ${(0, errors_1.getErrorMessage)(e)}`);
            }
        }
    });
    return npmDeps;
}
//# sourceMappingURL=packageJSON.js.map