"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateCompositeType = void 0;
const tslib_1 = require("tslib");
const lowcode_types_1 = require("@alilc/lowcode-types");
const lodash_1 = tslib_1.__importDefault(require("lodash"));
const types_1 = require("../types");
const jsExpression_1 = require("./jsExpression");
const jsSlot_1 = require("./jsSlot");
const aopHelper_1 = require("./aopHelper");
const expressionParser_1 = require("./expressionParser");
const common_1 = require("./common");
function isVariable(v) {
    if (lodash_1.default.isObject(v) && v.type === 'variable') {
        return true;
    }
    return false;
}
/**
 * 判断是否是数据源类型
 */
function isDataSource(v) {
    return typeof v === 'object' && v != null && v.type === 'DataSource';
}
function generateArray(value, scope, options = {}) {
    const body = value.map((v) => generateUnknownType(v, scope, options)).join(',');
    return `[${body}]`;
}
function generateObject(value, scope, options = {}) {
    if (value.type === 'i18n') {
        // params 可能会绑定变量，这里需要处理下
        if (value.params && typeof value.params === 'object') {
            return `this._i18nText(${generateUnknownType(lodash_1.default.omit(value, 'type'), scope, options)})`;
        }
        return `this._i18nText(${JSON.stringify(lodash_1.default.omit(value, 'type'))})`; // TODO: 优化：这里可以考虑提取成个常量...
    }
    const body = Object.keys(value)
        .map((key) => {
        const propName = JSON.stringify(key);
        const v = generateUnknownType(value[key], scope, options);
        return `${propName}: ${v}`;
    })
        .join(',\n');
    return `{${body}}`;
}
function generateString(value) {
    // 有的字符串里面会有特殊字符，比如换行或引号之类的，这里我们借助 JSON 的字符串转义功能来做下转义并加上双引号
    return JSON.stringify(value);
}
function generateNumber(value) {
    return String(value);
}
function generateBool(value) {
    return value ? 'true' : 'false';
}
function genFunction(value) {
    const globalVars = (0, expressionParser_1.parseExpressionGetKeywords)(value.value);
    if (globalVars.includes('arguments')) {
        return (0, jsExpression_1.generateFunction)(value, { isBindExpr: true });
    }
    return (0, jsExpression_1.generateFunction)(value, { isArrow: true });
}
function genJsSlot(value, scope, options = {}) {
    if (options.nodeGenerator) {
        return (0, jsSlot_1.generateJsSlot)(value, scope, options.nodeGenerator);
    }
    return '';
}
function generateUnknownType(value, scope, options = {}) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j;
    if (lodash_1.default.isUndefined(value)) {
        return 'undefined';
    }
    if (lodash_1.default.isNull(value)) {
        return 'null';
    }
    if (lodash_1.default.isArray(value)) {
        if ((_a = options.handlers) === null || _a === void 0 ? void 0 : _a.array) {
            return (0, aopHelper_1.executeFunctionStack)(value, scope, options.handlers.array, generateArray, options);
        }
        return generateArray(value, scope, options);
    }
    // FIXME: 这个是临时方案
    // 在遇到 type variable 私有类型时，转换为 JSExpression
    if (isVariable(value)) {
        const transValue = {
            type: 'JSExpression',
            value: value.variable,
        };
        if ((_b = options.handlers) === null || _b === void 0 ? void 0 : _b.expression) {
            const expression = (0, aopHelper_1.executeFunctionStack)(transValue, scope, options.handlers.expression, jsExpression_1.generateExpression, options);
            return expression || 'undefined';
        }
        return (0, jsExpression_1.generateExpression)(transValue, scope);
    }
    if ((0, lowcode_types_1.isJSExpression)(value)) {
        if ((_c = options.handlers) === null || _c === void 0 ? void 0 : _c.expression) {
            return (0, aopHelper_1.executeFunctionStack)(value, scope, options.handlers.expression, jsExpression_1.generateExpression, options);
        }
        return (0, jsExpression_1.generateExpression)(value, scope);
    }
    if ((0, lowcode_types_1.isJSFunction)(value) || (0, common_1.isJSExpressionFn)(value)) {
        if ((_d = options.handlers) === null || _d === void 0 ? void 0 : _d.function) {
            return (0, aopHelper_1.executeFunctionStack)(value, scope, options.handlers.function, genFunction, options);
        }
        return genFunction(value);
    }
    if ((0, lowcode_types_1.isJSSlot)(value)) {
        if ((_e = options.handlers) === null || _e === void 0 ? void 0 : _e.slot) {
            return (0, aopHelper_1.executeFunctionStack)(value, scope, options.handlers.slot, genJsSlot, options);
        }
        return genJsSlot(value, scope, options);
    }
    if (isDataSource(value)) {
        return generateUnknownType({
            type: 'JSExpression',
            value: `this.dataSourceMap[${JSON.stringify(value.id)}]`,
        }, scope, options);
    }
    if (lodash_1.default.isObject(value)) {
        if ((_f = options.handlers) === null || _f === void 0 ? void 0 : _f.object) {
            return (0, aopHelper_1.executeFunctionStack)(value, scope, options.handlers.object, generateObject, options);
        }
        return generateObject(value, scope, options);
    }
    if (lodash_1.default.isString(value)) {
        if ((_g = options.handlers) === null || _g === void 0 ? void 0 : _g.string) {
            return (0, aopHelper_1.executeFunctionStack)(value, scope, options.handlers.string, generateString, options);
        }
        return generateString(value);
    }
    if (lodash_1.default.isNumber(value)) {
        if ((_h = options.handlers) === null || _h === void 0 ? void 0 : _h.number) {
            return (0, aopHelper_1.executeFunctionStack)(value, scope, options.handlers.number, generateNumber, options);
        }
        return generateNumber(value);
    }
    if (lodash_1.default.isBoolean(value)) {
        if ((_j = options.handlers) === null || _j === void 0 ? void 0 : _j.boolean) {
            return (0, aopHelper_1.executeFunctionStack)(value, scope, options.handlers.boolean, generateBool, options);
        }
        return generateBool(value);
    }
    throw new types_1.CodeGeneratorError('Meet unknown composite value type');
}
// 这一层曾经是对产出做最外层包装的，但其实包装逻辑不应该属于这一层
// 这一层先不去掉，做冗余，方便后续重构
function generateCompositeType(value, scope, options = {}) {
    const result = generateUnknownType(value, scope, options);
    return result;
}
exports.generateCompositeType = generateCompositeType;
//# sourceMappingURL=compositeType.js.map