"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateJsSlot = void 0;
const lowcode_types_1 = require("@alilc/lowcode-types");
const types_1 = require("../types");
const jsxHelpers_1 = require("./jsxHelpers");
function generateSingleLineComment(commentText) {
    return `/* ${commentText.split('\n').join(' ').replace(/\*\//g, '*-/')}*/`;
}
function generateJsSlot(slot, scope, generator) {
    if ((0, lowcode_types_1.isJSSlot)(slot)) {
        const { title, params, value } = slot;
        // slot 也是分有参数和无参数的
        // - 有参数的 slot 就是类似一个 render 函数，需要创建子作用域
        // - 无参数的 slot 就是类似一个 JSX 节点，不需要创建子作用域
        const slotScope = params ? scope.createSubScope(params || []) : scope;
        const contentExpr = !value
            ? 'null'
            : generateNodeDataOrArrayForJsSlot(value, generator, slotScope);
        if (params) {
            return [
                title && generateSingleLineComment(title),
                '(',
                (params || []).join(', '),
                ') => ((__$$context) => (',
                contentExpr,
                '))(',
                `  __$$createChildContext(__$$context, { ${(params || []).join(', ')} }`,
                '))',
            ]
                .filter(Boolean)
                .join('');
        }
        return contentExpr || '[]';
    }
    throw new types_1.CodeGeneratorError('Not a JSSlot');
}
exports.generateJsSlot = generateJsSlot;
function generateNodeDataOrArrayForJsSlot(value, generator, scope) {
    if (Array.isArray(value)) {
        if (value.length === 0) {
            return '[]';
        }
        if (value.length === 1) {
            return (0, jsxHelpers_1.unwrapJsExprQuoteInJsx)(generator(value, scope)) || 'null';
        }
        return `[\n${value
            .map((v) => {
            if (typeof v === 'string') {
                return JSON.stringify(v);
            }
            return (0, jsxHelpers_1.unwrapJsExprQuoteInJsx)(generator(v, scope)) || 'null';
        })
            .join(',\n') || 'null'}\n]`;
    }
    return (0, jsxHelpers_1.unwrapJsExprQuoteInJsx)(generator(value, scope)) || 'null';
}
//# sourceMappingURL=jsSlot.js.map