"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isValidContainerType = exports.handleSubNodes = exports.isNpmInfo = exports.isContainerSchema = void 0;
const tslib_1 = require("tslib");
const _ = tslib_1.__importStar(require("lodash"));
const lowcode_types_1 = require("@alilc/lowcode-types");
const error_1 = require("../types/error");
const common_1 = require("./common");
function isContainerSchema(x) {
    return (typeof x === 'object' &&
        x &&
        typeof x.componentName === 'string' &&
        typeof x.fileName === 'string');
}
exports.isContainerSchema = isContainerSchema;
function isNpmInfo(x) {
    return typeof x === 'object' && x && typeof x.package === 'string';
}
exports.isNpmInfo = isNpmInfo;
const noop = () => undefined;
const handleChildrenDefaultOptions = {
    rerun: false,
};
const DEFAULT_MAX_DEPTH = 100000;
/**
 * 遍历并处理所有的子节点
 * @param children
 * @param handlers
 * @param options
 * @returns
 */
function handleSubNodes(children, handlers, options) {
    var _a;
    const opt = Object.assign(Object.assign({}, handleChildrenDefaultOptions), (options || {}));
    const maxDepth = (_a = opt.maxDepth) !== null && _a !== void 0 ? _a : DEFAULT_MAX_DEPTH;
    if (maxDepth <= 0) {
        throw new Error('handleSubNodes maxDepth reached');
    }
    if (Array.isArray(children)) {
        const list = children;
        return list
            .map((child) => handleSubNodes(child, handlers, Object.assign(Object.assign({}, opt), { maxDepth: maxDepth - 1 })))
            .reduce((p, c) => p.concat(c), []);
    }
    let result;
    const childrenRes = [];
    if (children === null || children === undefined) {
        return [];
    }
    else if ((0, lowcode_types_1.isDOMText)(children)) {
        const handler = handlers.string || noop;
        result = handler(children);
    }
    else if ((0, lowcode_types_1.isJSExpression)(children)) {
        const handler = handlers.expression || noop;
        result = handler(children);
    }
    else if ((0, lowcode_types_1.isJSSlot)(children)) {
        return handleSubNodes(children.value, handlers, Object.assign(Object.assign({}, opt), { maxDepth: maxDepth - 1 }));
    }
    else if ((0, lowcode_types_1.isNodeSchema)(children)) {
        const handler = handlers.node || noop;
        const child = children;
        result = handler(child);
        if (child.children) {
            const childRes = handleSubNodes(child.children, handlers, opt);
            childrenRes.push(...childRes);
        }
        if (child.props) {
            if (Array.isArray(child.props)) {
                child.props.forEach(({ value }) => {
                    const childRes = handleCompositeValueInProps(value);
                    childrenRes.push(...childRes);
                });
            }
            else {
                Object.values(child.props).forEach((value) => {
                    const childRes = handleCompositeValueInProps(value);
                    childrenRes.push(...childRes);
                });
            }
        }
    }
    else {
        throw new error_1.CodeGeneratorError('handleSubNodes got invalid NodeData', children);
    }
    if (result !== undefined) {
        childrenRes.unshift(result);
    }
    return childrenRes;
    function handleCompositeValueInProps(value) {
        if ((0, lowcode_types_1.isJSSlot)(value)) {
            return handleSubNodes(value.value, handlers, Object.assign(Object.assign({}, opt), { maxDepth: maxDepth - 1 }));
        }
        // CompositeArray
        if (Array.isArray(value)) {
            return _.flatMap(value, (v) => handleCompositeValueInProps(v));
        }
        // IPublicTypeCompositeObject
        if (!(0, lowcode_types_1.isJSExpression)(value) &&
            !(0, common_1.isJSExpressionFn)(value) &&
            !(0, lowcode_types_1.isJSFunction)(value) &&
            typeof value === 'object' &&
            value !== null) {
            return _.flatMap(Object.values(value), (v) => handleCompositeValueInProps(v));
        }
        return [];
    }
}
exports.handleSubNodes = handleSubNodes;
function isValidContainerType(schema) {
    return [
        'Page',
        'Component',
        'Block',
    ].includes(schema.componentName);
}
exports.isValidContainerType = isValidContainerType;
//# sourceMappingURL=schema.js.map